#include "StampRc.h"

#include <Bitmap.h>
#include <stdio.h>
#include <View.h>
#include <string.h>
#include <Region.h>
#include <String.h>
#include <Slider.h>
#include <Box.h>
#include <CheckBox.h>
#include <Button.h>
#include <StringView.h>
#include <File.h>
#include <FindDirectory.h>
#include <RadioButton.h>
#include <TextControl.h>

#include <stdlib.h>

status_t GetVideoPlugins(TList<VideoPluginType> & plugin_list, VideoPluginsHost * host, image_id id)
{
   status_t compatible = host->APIVersionCompatible();
   if (compatible != B_OK)
      return compatible;

   plugin_list.AddItem(new StampRcPluginType(host, id));
   return B_OK;
}

StampRcPluginType::StampRcPluginType(VideoPluginsHost * host, image_id id) :
VideoPluginType(host, id)
{
}

StampRcPluginType::~StampRcPluginType()
{
}

const char* StampRcPluginType::ID()
{
   return "StampRc";
}

uint32 StampRcPluginType::Flags()
{
   return VAPI_CONFIGURABLE;
}

uint32 StampRcPluginType::Version()
{
   return 001;
}

const char* StampRcPluginType::Name()
{
   return "StampRc";
}

const char* StampRcPluginType::Author()
{
   return "Zadig";
}

static const vertice_properties support[] =
{
   {
      ANY_COLOR_SPACE,
      ANY_COLOR_SPACE,
      1.0,
      VAPI_PROCESS_IN_PLACE | VAPI_BVIEW_DRAWING
   }
};

int32 StampRcPluginType::GetVerticesProperties(const vertice_properties ** ap)
{
   *ap = support;
   return sizeof(support) / sizeof(vertice_properties);
}

VideoPluginEngine * StampRcPluginType::InstanciateVideoPluginEngine()
{
   return new StampRcEngine(this);
}

//-------------------------------------------------------------------------
// This view is used as center of configuration.
// It handles the controls in it, and forwards the changes to the engine
class ConfigView : public BView {
   public:
      ConfigView(StampRcEngine * engine, BRect frame);
      virtual void AttachedToWindow();
      virtual void DetachedFromWindow();
      virtual void MessageReceived(BMessage *msg);
      void    Update();

   private:
      StampRcEngine * fEngine;
      BTextControl * fTextControl;
};

   ConfigView::ConfigView(StampRcEngine * engine, BRect frame) :
BView(frame, B_EMPTY_STRING, B_FOLLOW_LEFT_RIGHT, B_WILL_DRAW), fEngine(engine), fTextControl(NULL)
{
}

void ConfigView::AttachedToWindow()
{
   BRect frame = Bounds();
   SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
   frame.InsetBy(20, 20);
   BString  text;
   fEngine->GetText(text);
   BMessage *m = new BMessage('text');
   fTextControl = new BTextControl(frame, B_EMPTY_STRING, "Nb Presets:", text.String(), m, B_FOLLOW_ALL);
   fTextControl->SetDivider(70);
   AddChild(fTextControl);
   fTextControl->SetTarget(this);
}

void ConfigView::DetachedFromWindow()
{
   Update();
   fEngine->SetConfigWindow(NULL);
}

void ConfigView::MessageReceived(BMessage *msg)
{
   switch (msg->what) {
      case 'text':
         Update();
         break;
      default:
         BView::MessageReceived(msg);
   }
}

void ConfigView::Update()
{
   if (fTextControl)
      fEngine->SetText(fTextControl->Text());
}


//-------------------------------------------------------------------------

StampRcEngine::StampRcEngine(VideoPluginType * plugin) : VideoPluginEngine(plugin)
{
   int32 i_ChannelCount;
   BString name;

   if(Host()->GetChannel(i_CurChannel) != B_OK)
      i_CurChannel = 1;
   else
      i_CurChannel++;

   i_ChannelCount = 6;
   fText = "6";
   
   pc_Reader = new RcReader(i_CurChannel, i_ChannelCount);
}

StampRcEngine::~StampRcEngine()
{   
   delete pc_Reader;
}

status_t StampRcEngine::UseFormat(media_raw_video_format & format, int which_work_mode)
{
   return B_OK;
}
status_t StampRcEngine::RestoreConfig(BMessage & config)
{
   const char * text;
   if (config.FindString("nb_channel", &text) == B_OK)
   {
      pc_Reader->SetChannelNb(atoi(text));
      SetText(text);
   }
   return B_OK;
}

status_t StampRcEngine::StoreConfig(BMessage & config)
{
   config.AddString("nb_channel", fText.String());
   return B_OK;
}

BView* StampRcEngine::ConfigureView()
{
   return new ConfigView(this, BRect(0, 0, 200, 50));
}

status_t StampRcEngine::ApplyEffect(BBitmap * frame_in, BBitmap * frame_out, int64 frame_count, bool skipped_frames)
{
   if(frame_in != frame_out)
      return(B_ERROR);

   if(pc_Reader->ChannelHasChanged((uint16*)&i_CurChannel))
   {
      Host()->SwitchToPreset(i_CurChannel-1);
   }
   /* memcpy(frame_out->Bits(), frame_in->Bits(), frame_in->BitsLength()); */
   return B_OK;
}

void StampRcEngine::GetText(BString	& text)
{
   text = fText;
}

void StampRcEngine::SetText(const char * text)
{
   fText = text;
   pc_Reader->SetChannelNb(atoi(text));
}
